==> jangaroo-compiler-0.8.6-sources.jar: net/jangaroo/jooc/ast/IdeExpr.java <==
/*
 * Copyright 2008 CoreMedia AG
 * 
 * Licensed under the Apache License, Version 2.0 (the "License"); 
 * you may not use this file except in compliance with the License. 
 * You may obtain a copy of the License at
 * http://www.apache.org/licenses/LICENSE-2.0 
 * 
 * Unless required by applicable law or agreed to in writing, 
 * software distributed under the License is distributed on an "AS
 * IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either 
 * express or implied. See the License for the specific language 
 * governing permissions and limitations under the License.
 */

package net.jangaroo.jooc.ast;

import net.jangaroo.jooc.JooSymbol;
import net.jangaroo.jooc.Scope;

import java.io.IOException;

/**
 * @author Andreas Gawecki
 */
public class IdeExpr extends Expr {

  private Ide ide;

  public IdeExpr(JooSymbol symIde) {
    this(new Ide(symIde));
  }

  public IdeExpr(Ide ide) {
    this.ide = ide;
  }

  public static IdeExpr fromPrefix(JooSymbol symPrefix, JooSymbol symDot, Ide ide) {
    return new IdeExpr(ide.qualify(symPrefix, symDot));
  }

  @Override
  public void visit(AstVisitor visitor) throws IOException {
    visitor.visitIdeExpression(this);
  }

  @Override
  public void scope(final Scope scope) {
    getIde().scope(scope);
  }

  @Override
  public void analyze(AstNode parentNode) {
    super.analyze(parentNode);
    getIde().analyze(this);
    getIde().analyzeAsExpr(parentNode, this);
    setType(getIde().resolveDeclaration());
  }

  public JooSymbol getSymbol() {
    return getIde().getSymbol();
  }

  @Override
  public boolean isCompileTimeConstant() {
    IdeDeclaration ideDeclaration = getIde().getDeclaration(false);
    // accept constant fields, not being defined in the current class (because these have to be initialized first):
    return ideDeclaration instanceof VariableDeclaration &&
            ((VariableDeclaration) ideDeclaration).isCompileTimeConstant() &&
            ideDeclaration.getClassDeclaration() != getIde().getScope().getClassDeclaration();
  }

  public Ide getIde() {
    return ide;
  }

}

==> jangaroo-compiler-0.9.4-sources.jar: net/jangaroo/jooc/ast/IdeExpr.java <==
/*
 * Copyright 2008 CoreMedia AG
 * 
 * Licensed under the Apache License, Version 2.0 (the "License"); 
 * you may not use this file except in compliance with the License. 
 * You may obtain a copy of the License at
 * http://www.apache.org/licenses/LICENSE-2.0 
 * 
 * Unless required by applicable law or agreed to in writing, 
 * software distributed under the License is distributed on an "AS
 * IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either 
 * express or implied. See the License for the specific language 
 * governing permissions and limitations under the License.
 */

package net.jangaroo.jooc.ast;

import net.jangaroo.jooc.JooSymbol;
import net.jangaroo.jooc.Scope;

import java.io.IOException;

/**
 * @author Andreas Gawecki
 */
public class IdeExpr extends Expr {

  private Ide ide;

  public IdeExpr(JooSymbol symIde) {
    this(new Ide(symIde));
  }

  public IdeExpr(Ide ide) {
    this.ide = ide;
  }

  public static IdeExpr fromPrefix(JooSymbol symPrefix, JooSymbol symDot, Ide ide) {
    return new IdeExpr(ide.qualify(symPrefix, symDot));
  }

  @Override
  public void visit(AstVisitor visitor) throws IOException {
    visitor.visitIdeExpression(this);
  }

  @Override
  public void scope(final Scope scope) {
    getIde().scope(scope);
  }

  @Override
  public void analyze(AstNode parentNode) {
    super.analyze(parentNode);
    getIde().analyze(this);
    getIde().analyzeAsExpr(parentNode, this);
    setType(getIde().resolveDeclaration());
  }

  public JooSymbol getSymbol() {
    return getIde().getSymbol();
  }

  @Override
  public boolean isCompileTimeConstant() {
    IdeDeclaration ideDeclaration = getIde().getDeclaration(false);
    // accept top-level get functions:
    if (ideDeclaration instanceof FunctionDeclaration &&
      ((FunctionDeclaration)ideDeclaration).isGetter()) {
      AstNode parentDeclaration = ideDeclaration.getParentDeclaration();
      if (parentDeclaration instanceof PackageDeclaration &&
        "".equals(((PackageDeclaration)parentDeclaration).getName())) {
        return true;
      }
    }
    // accept constant fields, not being defined in the current class (because these have to be initialized first):
    return ideDeclaration instanceof VariableDeclaration &&
            ((VariableDeclaration) ideDeclaration).isCompileTimeConstant() &&
            ideDeclaration.getClassDeclaration() != getIde().getScope().getClassDeclaration();
  }

  public Ide getIde() {
    return ide;
  }

}
